searchNodes=[{"ref":"FreeAst.html","title":"FreeAst","type":"module","doc":"Implements something similar to Free Monad. Usage FreeAst.p/1 macro returns a program that can be interpreted later. Expressions to interpret are specified with effect/2 and eval/1 calls. The first one is used to specify a desired effect, while the second one -- to evaluate another program. Here we define two functions that return programs with effects: defmodule FreeAstExample.Reader do @moduledoc false require FreeAst def read_line do FreeAst.p do effect IO, read_line() end end end defmodule FreeAstExample.Greater do @moduledoc false require FreeAst alias FreeAstExample.Reader def great do FreeAst.p do greating = effect Environment, greating() greating &lt;&gt; &quot;, &quot; &lt;&gt; eval Reader.read_line() end end end Now we can define interpreters for these: defmodule FreeAstExample.IOInterpreter do @moduledoc false def interpret(IO, :read_line, []), do: &quot;John&quot; end defmodule FreeAstExample.EnvironmentInterpreter do @moduledoc false def interpret(Environment, :greating, []), do: &quot;Hi&quot; end For our first program we can use the first interpreter, but for the second one we need both, so we compose interpreters: defmodule FreeAstExample.MainInterpreter do @moduledoc false alias FreeAst.Interpreter alias FreeAstExample.{IOInterpreter, EnvironmentInterpreter} def interpreter do Interpreter.noop() |&gt; Interpreter.compose(IO, &amp;IOInterpreter.interpret/3) |&gt; Interpreter.compose(Environment, &amp;EnvironmentInterpreter.interpret/3) end end So now we can execute it with FreeAst.interpret/2: iex(1)&gt; alias FreeAstExample.{Greater, MainInterpreter} iex(2)&gt; FreeAst.interpret(Greater.great(), MainInterpreter.interpreter()) &quot;Hi, John&quot;"},{"ref":"FreeAst.html#defast/2","title":"FreeAst.defast/2","type":"macro","doc":"Short syntax for defining functions that return programs defmodule FreeAstExample.DefastExample do @moduledoc false import FreeAst, only: [defast: 2] require FreeAst defast program_v1 do :value end def program_v2 do FreeAst.p do :value end end end iex(1)&gt; import FreeAstExample.DefastExample iex(2)&gt; import FreeAst.Interpreter, only: [noop: 0] iex(3)&gt; result_v1 = FreeAst.interpret(program_v1(), noop()) iex(4)&gt; result_v2 = FreeAst.interpret(program_v2(), noop()) iex(5)&gt; result_v1 == result_v2 true"},{"ref":"FreeAst.html#interpret/2","title":"FreeAst.interpret/2","type":"function","doc":"Interprets a program with the supplied interpreter."},{"ref":"FreeAst.html#p/1","title":"FreeAst.p/1","type":"macro","doc":"Creates a program from the supplied code block. iex(1)&gt; program = ...(1)&gt; FreeAst.p do ...(1)&gt; x = effect IO, read_line() ...(1)&gt; &quot;Hi, &quot; &lt;&gt; x ...(1)&gt; end iex(2)&gt; FreeAst.interpret(program, fn IO, :read_line, [] -&gt; &quot;Kek&quot; end) &quot;Hi, Kek&quot;"},{"ref":"FreeAst.html#t:interpreter/0","title":"FreeAst.interpreter/0","type":"type","doc":"Interpreter is a function that takes three arguments: action kind which is an atom or an alias, action name which is an atom, and a list of arguments of the action."},{"ref":"FreeAst.html#t:program/0","title":"FreeAst.program/0","type":"opaque","doc":"Represents a program (returned by p/1 macro)."},{"ref":"FreeAst.Interpreter.html","title":"FreeAst.Interpreter","type":"module","doc":"Helpers to work with interpreters."},{"ref":"FreeAst.Interpreter.html#compose/3","title":"FreeAst.Interpreter.compose/3","type":"function","doc":"Composes interpreters for different effect kinds Suppose you have two interpreters for two kinds of actions: defmodule MyApp.FileEffects do def interpret(_, :ls, [path]), do: File.ls(path) end defmodule MyApp.EnvEffects do def interpret(_, :some_variable, []) do Application.get_env(:my_app, :some_variable) end end Now you can compose it like this: defmodule MyApp.Effects do alias MyApp.{EnvEffects, FileEffects} def interpret(kind, action, attrs) do Effect.noop() |&gt; Effect.compose(:file, &amp;FileEffects.interpret/3) |&gt; Effect.compose(:env, &amp;EnvEffects.interpret/3) |&gt; apply([kind, action, attrs]) end end"},{"ref":"FreeAst.Interpreter.html#noop/0","title":"FreeAst.Interpreter.noop/0","type":"function","doc":"Empty effect interpreter; may be used to start effect interpreter composition See compose/3 for more details."},{"ref":"readme.html","title":"FreeAst","type":"extras","doc":"FreeAst Implements something similar to Free Monad. See examples directory for more information."},{"ref":"readme.html#usage","title":"FreeAst - Usage","type":"extras","doc":"See hexdocs documentation for details."},{"ref":"readme.html#installation","title":"FreeAst - Installation","type":"extras","doc":"Add the following code to your mix.exs: def deps do [ {:free_ast, &quot;~&gt; 0.3&quot;} ] end"}]